# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.ConnectionState.Handshake do
  require Amethyst.ConnectionState.Macros
  alias Amethyst.ConnectionState.Macros

  require Logger

  @moduledoc """
  This module contains the packets and logic for the Handshake state.
  """
  # Notice that in the Handshake state, our version is always 0.
  Macros.defpacket_serverbound :handshake, 0x00, 0, [
    version: :varint,
    address: :string,
    port: :ushort,
    next: :varint
  ]

  def handle(%{packet_type: :handshake, version: 767, address: address, port: port, next: next}, 0, _state) do
    Logger.debug("Received handshake for #{address}:#{port} with version 767")
    case next do
      1 ->
        send(self(), {:set_state, Amethyst.ConnectionState.Status})
        send(self(), {:set_version, 767})
        :ok
      2 ->
        send(self(), {:set_state, Amethyst.ConnectionState.Login})
        send(self(), {:set_version, 767})
        :ok
      3 ->
        send(self(), {:set_state, Amethyst.ConnectionState.Transfer})
        send(self(), {:set_version, 767})
        :ok
      _ -> {:error, "Invalid next state"}
    end
  end

  def disconnect(_reason) do
    # When disconnecting from the handshake state, we can't send any sort of reason.
  end
end
