# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.ConnectionState.Status do
  require Amethyst.ConnectionState.Macros
  alias Amethyst.ConnectionState.Macros

  require Logger

  @moduledoc """
  This module contains the packets and logic for the Status state.
  """
  Macros.defpacket_clientbound :status_response, 0x00, 767, [json_response: :string]
  Macros.defpacket_clientbound :pong_response, 0x01, 767, [payload: :long]

  Macros.defpacket_serverbound :status_request, 0x00, 767, []
  Macros.defpacket_serverbound :ping_request, 0x01, 767, [payload: :long]

  def handle(%{packet_type: :status_request}, 767, _state) do
    Logger.debug("Received status request")
    send(self(), {:send_packet, %{
      packet_type: :status_response,
      json_response: ~s({
"version": {"name": "1.21", "protocol": 767},
"players": {"max": -1, "online": 69, "sample": [{"name": "§dAmethyst§r", "id": "4566e69f-c907-48ee-8d71-d7ba5aa00d20"}]},
"description": {"text":"Amethyst is an experimental server written in Elixir"},
"enforcesSecureChat": false,
"previewsChat": false,
"preventsChatReports": true
})
    }})
    :ok
  end

  def handle(%{packet_type: :ping_request, payload: payload}, 767, _state) do
    Logger.debug("Received ping request")
    send(self(), {:send_packet, %{
      packet_type: :pong_response,
      payload: payload
    }})
    :ok
  end

  def disconnect(_reason) do
    # When disconnecting from the status state, we can't send any sort of reason.
  end
end
